@@ -1,8 +1,6 @@
 /*
  * Copyright (c) 2010 David Galindo <nowheremanmail@gmail.com>
 
-  // TODO  several files? or several wallets in database?
-
  */
 
 #include "Constants.h"
@@ -61,30 +59,33 @@ bool Target::existField (QString name) {
 	}
 	return false;
 }
-
+QString Target::getSummary () {
+	return this->type;
+}
 QImage * Target::getPictureSmall () {
+	//qDebug () << "getPictureSmall " << this->id << " " << pictureSmall;
 	if(!pictureSmall) {
 		QImage * orig = getPicture();
 
 		if (orig) {
-			pictureSmall = new QImage ();
-			*pictureSmall = orig->scaled (SMALL_WIDTH, SMALL_HEIGH);
+			pictureSmall = new QImage (orig->scaled (SMALL_WIDTH, SMALL_HEIGH));
 		}
 		else
 			pictureSmall = NULL;
 
-
 	}
 	return pictureSmall;
 }
 QImage * Target::getPicture () {
-	if(!picture)
+	if(!picture) {
 		picture = ::dbManager->getPicture (this);
+	}
 	return picture;
 }
 QImage * Target::getPictureBack () {
-	if(!pictureBack)
+	if(!pictureBack) {
 		pictureBack = ::dbManager->getPictureBack (this);
+	}
 	return pictureBack;
 }
 /*void Target::setPictureSmall (QImage * a) {
@@ -95,6 +96,7 @@ void Target::setPicture (QImage * a) {
 	if(picture) delete picture;
 	picture = a;
 	if(pictureSmall) delete pictureSmall;
+	pictureSmall = NULL;
 }
 void Target::setPictureBack (QImage * a) {
 	if(pictureBack) delete pictureBack;
@@ -542,7 +544,7 @@ QList<Target*> DatabaseManager::getTargets(QString filter)
 {
 	QList<Target*> result;
 
-	QSqlQuery query("select id, name, type from targets");
+	QSqlQuery query("select id, name, type from targets order by name");
 	//QSqlQuery query("select id, name, type, picture, pictureBack  from targets");
 	while (query.next()) {
 		Target * item = new Target();
@@ -730,7 +732,8 @@ bool DatabaseManager::updateTargetField(TargetField* field)
 				encrypt(field->value, b);
 				query.bindValue(":name", a);
 				query.bindValue(":value", b);
-			} else {
+			}
+			else {
 				query.bindValue(":name", field->name);
 				query.bindValue(":value", field->value);
 			}
@@ -764,7 +767,8 @@ bool DatabaseManager::insertTargetField(TargetField* field)
 				encrypt(field->value, b);
 				query.bindValue(":name", a);
 				query.bindValue(":value", b);
-			} else {
+			}
+			else {
 				query.bindValue(":name", field->name);
 				query.bindValue(":value", field->value);
 			}
@@ -803,7 +807,8 @@ void DatabaseManager::getFields(Target * target)
 		if (item->encrypted) {
 			decrypt(query.value(1).toByteArray(), item->name);
 			decrypt(query.value(2).toByteArray(), item->value);
-		} else {
+		}
+		else {
 			item->name = query.value(1).toString();
 			item->value= query.value(2).toString();
 		}
@@ -820,14 +825,16 @@ QImage * DatabaseManager::getPicture (Target * target) {
 	while (query.next()) {
 		if (query.value(0).isNull()) {
 			result = NULL;
-		} else {
+		}
+		else {
 			result = new QImage ();
 			QByteArray im_s;
 			decrypt(query.value(0).toByteArray(), im_s);
 			if (!result->loadFromData (im_s)) {
 				qDebug () << "Error loading image " << target->id;
-			} else
+			} else {
 				qDebug () << "loaded image " << target->id;
+			}
 		}
 		break;
 	}
@@ -843,14 +850,17 @@ QImage * DatabaseManager::getPictureBack (Target * target) {
 	while (query.next()) {
 		if (query.value(0).isNull()) {
 			result = NULL;
-		} else {
+		}
+		else {
 			result = new QImage ();
 			QByteArray im_s;
 			decrypt(query.value(0).toByteArray(), im_s);
 			if (!result->loadFromData (im_s)) {
 				qDebug () << "Error loading back image " << target->id;
-			} else
-				qDebug () << "loaded back image " << target->id;
+			}
+			else {
+				//qDebug () << "loaded back image " << target->id;
+			}
 		}
 		break;
 	}
