@@ -27,6 +27,7 @@ public:
 	~TargetField();
 
 	TargetField(QString name, int type, bool encrypted, int order) {
+		this->id = -1;
 		this->name = name;
 		this->type = type;
 		this->encrypted = encrypted;
@@ -64,6 +65,7 @@ public:
 
 	void freeResources ();
 	void freeFieldList ();
+	QString getSummary ();
 
 	bool existField (QString name);
 //private:
