@@ -11,6 +11,8 @@
 //#include <QXmlSimpleReader>
 
 
+// TODO after insert a field with password, then click to see, crash when pushing password button
+
 DetailWindow::DetailWindow(Target * target, bool edit, eWallet *paTarget, QWidget *parent) :
 		QMainWindow(parent)
 {
@@ -26,6 +28,8 @@ DetailWindow::DetailWindow(Target * target, bool edit, eWallet *paTarget, QWidge
 		changeFields (0);
 	}
 
+	qDebug () << " going to " << edit << " " << (isNew ? "NEW" : target->name);
+
 	orgType = this->target->type;
 
 	deleteFace = false;
@@ -49,9 +53,14 @@ DetailWindow::DetailWindow(Target * target, bool edit, eWallet *paTarget, QWidge
 		if (!isNew)
 			menuBar()->addAction("Delete", this, SLOT(_deleteTarget()));
 		menuBar()->addAction("Change Face Pict.", this, SLOT(cardFace()));
+		menuBar()->addAction("Mirror Face Pict.", this, SLOT(mirrorFace()));
 		menuBar()->addAction("Change Back Pict.", this, SLOT(cardBack()));
+		menuBar()->addAction("Mirror Back Pict.", this, SLOT(mirrorBack()));
 		menuBar()->addAction("Delete Face Pict.", this, SLOT(deleteCardFace()));
 		menuBar()->addAction("Delete Back Pict.", this, SLOT(deleteCardBack()));
+		menuBar()->addAction("Modify field", this, SLOT(modifyFields()));
+		//TODO menuBar()->addAction("Delete field", this, SLOT(addNewField()));
+		//TODO menuBar()->addAction("Modify field", this, SLOT(addNewField()));
 	}
 	else {
 		menuBar()->addAction("New", this, SLOT(_newTarget()));
@@ -67,20 +76,10 @@ DetailWindow::DetailWindow(Target * target, bool edit, eWallet *paTarget, QWidge
 	vboxlayout = new QVBoxLayout ();
 	vboxlayout->setSpacing(0);
 	vboxlayout->setMargin(0);
-	// Custom list box
-	//fnListWidget = new QFNListWidget(this);
-	//vboxlayout->addWidget(fnListWidget);
-
-	//QFrame * entryFrame = new QFrame(this);
-	//entryFrame->setContextMenuPolicy(Qt::NoContextMenu);
 
 	entryScrollArea = new QScrollArea;
 	//entryScrollArea->setProperty("FingerScrollable", false);
 	entryScrollArea->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
-	//entryFrame->setProperty("FingerScrollable", false);
-	//entryScrollArea->setPalette(*p);
-	//entryScrollArea->setFocusPolicy(Qt::NoFocus);
-	//entryScrollArea->setContextMenuPolicy(Qt::NoContextMenu);
 	entryScrollArea->setWidgetResizable(true);
 	entryScrollArea->setWidget(centralWidget);
 	entryScrollArea->setProperty("FingerScrollable", true);
@@ -90,27 +89,18 @@ DetailWindow::DetailWindow(Target * target, bool edit, eWallet *paTarget, QWidge
 
 	// Create layout for the ui controls
 	componentLayout = new QGridLayout ();
-	//componentLayout->setContentsMargins(15, 0, 15, 0);
-	//componentLayout->setSpacing(5);
 	// Name label
 	if (edit) {
 		QLabel * nameLabel = new QLabel("Name", this);
-		//nameLabel->setContextMenuPolicy(Qt::NoContextMenu);
 		componentLayout->addWidget(nameLabel, 1, 1, 1, 2);
 
 	    nameLineEdit = new QLineEdit(this);
-		//nameLineEdit->setContextMenuPolicy(Qt::NoContextMenu);
-		//nameLineEdit->setAlignment(Qt::AlignTop);
 		componentLayout->addWidget(nameLineEdit, 1, 3, 1, 4);
 		if (target)
 			nameLineEdit->setText (target->name);
 	}
 	else {
 		this->setWindowTitle (target->name);
-		/*
-		QLabel * nameLineEdit = new QLabel(target->name, this);
-		nameLineEdit->setContextMenuPolicy(Qt::NoContexnameLineEdittMenu);
-		componentLayout->addWidget(nameLineEdit, 1, 3, 1, 4);*/
 	}
 
 	if (edit) {
@@ -136,21 +126,10 @@ DetailWindow::DetailWindow(Target * target, bool edit, eWallet *paTarget, QWidge
 
 	    connect(typeLineEdit, SIGNAL(activated(int)), this, SLOT(emitCommitData(int)));
 
-
-		//typeLineEdit->setContextMenuPolicy(Qt::NoContextMenu);
-		//typeLineEdit->setAlignment(Qt::AlignTop);
 		componentLayout->addWidget(typeLineEdit, 2, 3, 1, 4);
-		//typeLineEdit->setText (target->type);
 	}
 	else {
-/*		// type label
-		QLabel * typeLabel = new QLabel("Type", this);
-		typeLabel->setContextMenuPolicy(Qt::NoContextMenu);
-		componentLayout->addWidget(typeLabel, 2, 1, 1, 2);
 
-		QLabel * typeLineEdit = new QLabel (target->type, this); //QLineEdit(this);
-		typeLineEdit->setContextMenuPolicy(Qt::NoContextMenu);
-		componentLayout->addWidget(typeLineEdit, 2, 3, 1, 4);*/
 	}
 
 	vboxlayout->addLayout(componentLayout);
@@ -158,15 +137,22 @@ DetailWindow::DetailWindow(Target * target, bool edit, eWallet *paTarget, QWidge
 	componentLayout = new QGridLayout ();
 
 	setContextMenuPolicy(Qt::CustomContextMenu);
-	QObject::connect(this, SIGNAL(customContextMenuRequested(const QPoint&)), this, SLOT(cardMenu(const QPoint&)));
-	//QObject::connect(centralWidget, SIGNAL(customContextMenuRequested(const QPoint&)), this, SLOT(cardFaceMenu(const QPoint&)));
+	//QObject::connect(this, SIGNAL(customContextMenuRequested(const QPoint&)), this, SLOT(cardMenu(const QPoint&)));
 
 	if (edit || target->getPicture() != NULL) {
 		imageLabel = new QLabel(this);
 		imageLabel->setContextMenuPolicy(Qt::CustomContextMenu);
 		imageLabel->setBackgroundRole(QPalette::NoRole);
-		imageLabel->setScaledContents(true);
-		if (edit) imageLabel->setMaximumSize(250, 200);
+		imageLabel->setContentsMargins(5, 2, 0, 2);
+		if (edit) {
+			imageLabel->setMaximumSize(BIG_WIDTH, BIG_HEIGH);
+			imageLabel->setMinimumSize(BIG_WIDTH, BIG_HEIGH);
+			imageLabel->setScaledContents(true);
+		}
+		else {
+			//imageLabel->setMaximumSize(MAX_WIDTH, MAX_HEIGH);
+			imageLabel->setScaledContents(true);
+		}
 		if (currentImage == NULL) {
 			QPixmap available(":/resources/empty_card.png");
 			imageLabel->setPixmap(available);
@@ -190,9 +176,17 @@ DetailWindow::DetailWindow(Target * target, bool edit, eWallet *paTarget, QWidge
 		imageBackLabel = new QLabel(this);
 		imageBackLabel->setContextMenuPolicy(Qt::CustomContextMenu);
 		imageBackLabel->setBackgroundRole(QPalette::NoRole);
-		imageBackLabel->setScaledContents(true);
 		//imageBackLabel->setFocusPolicy(Qt::StrongFocus);
-		if (edit) imageBackLabel->setMaximumSize(250, 200);
+		imageBackLabel->setContentsMargins(5, 2, 0, 2);
+		if (edit) {
+			imageBackLabel->setMaximumSize(BIG_WIDTH, BIG_HEIGH);
+			imageBackLabel->setMinimumSize(BIG_WIDTH, BIG_HEIGH);
+			imageBackLabel->setScaledContents(true);
+		}
+		else {
+			//imageBackLabel->setMaximumSize(MAX_WIDTH, MAX_HEIGH);
+			imageBackLabel->setScaledContents(true);
+		}
 		if (currentImageBack == NULL) {
 			QPixmap availableBack(":/resources/empty_card_back.png");
 			imageBackLabel->setPixmap(availableBack);
@@ -217,16 +211,85 @@ DetailWindow::DetailWindow(Target * target, bool edit, eWallet *paTarget, QWidge
 		componentLayout = new QGridLayout ();
 	}
 
+	componentLayout->setColumnMinimumWidth(1, 100);
+
 	showFields ();
 
 	vboxlayout->addLayout(componentLayout);
 }
 
+void DetailWindow::modifyFields() {
+	if (createFieldDlg.isNull()) {
+		createFieldDlg = new CreateFieldDialog(this->target, this);
+		createFieldDlg->setMinimumSize(size().width(),size().height()-100);
+		createFieldDlg->exec();
+
+		qDebug () << "NOW ";
+
+		showFields ();
+
+		delete createFieldDlg;
+		createFieldDlg = 0;
+	}
+
+}
+
+void DetailWindow::addField (TargetField * field) {
+	target->fields.append(field);
+
+	_addField (field);
+}
+
+void DetailWindow::_addField (TargetField * field) {
+	int s = 1;
+	if (field->type == TYPE_TEXT) {
+		s = 10;
+	}
+
+	//qDebug () << "trying to show " << field->name << " v=" << field->value << " t=" << field->type;
+
+	QLineEdit * nameLabel = new QLineEdit(this);
+	//nameLabel->setContextMenuPolicy(Qt::NoContextMenu);
+	componentLayout->addWidget(nameLabel, pos + 1, 1, 1, 2);
+	nameLabel->setText (field->name);
+	this->labels.append (nameLabel);
+
+	//ButtonInfo * info = new ButtonInfo (field);
+	//nameLabel->setUserData(0, info);
+
+	if (field->type == TYPE_TEXT) {
+		QTextEdit * nameLineEdit = new QTextEdit(this);
+		//nameLineEdit->setContextMenuPolicy(Qt::NoContextMenu);
+		componentLayout->addWidget(nameLineEdit, pos + 1, 3, s, 4);
+		nameLineEdit->setText (field->value);
+		this->values.append (nameLineEdit);
+		//nameLineEdit->setUserData(0,info);
+	}
+/*	else
+	if (field->type == TYPE_PHONE_NUMBER) {
+		// TODO
+		QLineEdit * nameLineEdit = new QLineEdit(this);
+		//nameLineEdit->setContextMenuPolicy(Qt::NoContextMenu);
+		componentLayout->addWidget(nameLineEdit, pos + 1, 3, s, 4);
+		nameLineEdit->setText (field->value);
+		this->values.append (nameLineEdit);
+
+	}*/
+	else {
+		QLineEdit * nameLineEdit = new QLineEdit(this);
+		//nameLineEdit->setContextMenuPolicy(Qt::NoContextMenu);
+		componentLayout->addWidget(nameLineEdit, pos + 1, 3, s, 4);
+		nameLineEdit->setText (field->value);
+		this->values.append (nameLineEdit);
+
+		//nameLineEdit->setUserData(0,info);
+	}
+
+}
+
 void DetailWindow::showFields () {
 
 	if (edit) {
-		//QObjectList list = componentLayout->children();
-
 		for (int i = 0; i < this->labels.size(); i++) {
 			componentLayout->removeWidget(this->labels[i]);
 			delete this->labels[i];
@@ -237,69 +300,75 @@ void DetailWindow::showFields () {
 			delete this->values[i];
 		}
 
-		//qDebug () << " list elements " << list.size();
-
 		this->values.clear();
 		this->labels.clear();
 	}
 
-	int pos = 0;
+	QSignalMapper * signalMapper = NULL;
+	if (!edit) {
+		signalMapper = new QSignalMapper(this);
+		QObject::connect(signalMapper, SIGNAL(mapped(QWidget *)), this, SLOT(clickPass(QWidget *)));
+	}
+
+	pos = 0;
 	if (target) {
 		TargetField * field;
 		foreach(field, target->fields) {
+			int s = 1;
+			if (field->type == TYPE_TEXT) {
+				s = 10;
+			}
 			if (edit) {
-	/*			QLabel * nameLabel = new QLabel(field->name, this);
-				nameLabel->setContextMenuPolicy(Qt::NoContextMenu);
-				componentLayout->addWidget(nameLabel, pos+1, 1, 1, 2);
-*/
-				QLineEdit * nameLabel = new QLineEdit(this);
-				//nameLabel->setContextMenuPolicy(Qt::NoContextMenu);
-				componentLayout->addWidget(nameLabel, pos + 1, 1, 1, 2);
-				nameLabel->setText (field->name);
-
-				QLineEdit * nameLineEdit = new QLineEdit(this);
-				//nameLineEdit->setContextMenuPolicy(Qt::NoContextMenu);
-				componentLayout->addWidget(nameLineEdit, pos + 1, 3, 1, 4);
-				nameLineEdit->setText (field->value);
-				
-				this->labels.append (nameLabel);
-				this->values.append (nameLineEdit);
+				_addField (field);
 			}
 			else {
 				if (field->value.length() > 0) {
 					QLabel * nameLabel = new QLabel(field->name, this);
+					nameLabel->setFont (QFont("arial", 11, 1, true));
+					nameLabel->setAlignment(Qt::AlignRight | Qt::AlignVCenter);
+					nameLabel->setContentsMargins(0, 0, 10, 0);
 					//nameLabel->setContextMenuPolicy(Qt::NoContextMenu);
 					componentLayout->addWidget(nameLabel, pos + 1, 1, 1, 2);
 
-					QLabel * nameLineEdit = new QLabel(field->value, this);
-					//nameLineEdit->setContextMenuPolicy(Qt::NoContextMenu);
-					componentLayout->addWidget(nameLineEdit, pos + 1, 3, 1, 4);
+					if (field->type == TYPE_PASSWORD || field->name.contains("password", Qt::CaseInsensitive)) {
+						QPushButton * nameLineEdit = new QPushButton("Click to see", this);
+						//nameLineEdit->setContextMenuPolicy(Qt::NoContextMenu);
+						componentLayout->addWidget(nameLineEdit, pos + 1, 3, s, 2);
 
-					//this->labels.append (nameLabel);
-					//this->labels.append (nameLineEdit);
+						nameLineEdit->setUserData(0, new ButtonInfo (field)); //ObjectName(field->value);
+						signalMapper->setMapping(nameLineEdit, nameLineEdit);
+
+						QObject::connect(nameLineEdit, SIGNAL(clicked()), signalMapper, SLOT(map()));
+					}
+					else {
+						QLabel * nameLineEdit = new QLabel(field->value, this);
+						//nameLineEdit->setContextMenuPolicy(Qt::NoContextMenu);
+						componentLayout->addWidget(nameLineEdit, pos + 1, 3, s, 4);
+					}
 				}
 			}
-			pos++;
+			pos += s;
 		}
-	}
-	
-/*	if (edit) {
-		for (int i = 0; i < 5; i++) {
-			QLineEdit * nameLabel = new QLineEdit(this);
-			nameLabel->setContextMenuPolicy(Qt::NoContextMenu);
-			componentLayout->addWidget(nameLabel, pos+1, 1, 1, 2);
-			//nameLabel->setText (field->name);
-			
-			QLineEdit * nameLineEdit = new QLineEdit(this);
-			nameLineEdit->setContextMenuPolicy(Qt::NoContextMenu);
-			componentLayout->addWidget(nameLineEdit, pos+1, 3, 1, 4);
-			//nameLineEdit->setText (field->value);
 
-			this->labels.append (nameLabel);
-			this->values.append (nameLineEdit);
-			pos++;
-		}
-	}*/
+	}
+
+	//qDebug () << " shown " << pos << " rows";
+	if (pos < 20) {
+		componentLayout->addItem(new QSpacerItem(1,20 - pos), pos + 1, 1, 20-pos, 6);
+		//qDebug () << " add " << (20 - pos) << " rows";
+	}
+}
+
+void DetailWindow::clickPass (QWidget * _wid) {
+	QPushButton * b = (QPushButton *)_wid;
+	//qDebug () << "click on field " << b->userData(0);
+	TargetField * f = ((ButtonInfo *)b->userData(0))->field;
+	if (b->text() == f->value) {
+		b->setText("Click to see");
+	}
+	else {
+		b->setText(f->value);
+	}
 }
 
 void DetailWindow::changeFields (int a) {
@@ -321,12 +390,7 @@ void DetailWindow::changeFields (int a) {
 		if (!target->existField (tmp.name)) {
 			TargetField * f;
 
-			f = new TargetField ();
-			f->name = tmp.name;
-			f->encrypted = tmp.encrypted;
-			f->value = "";
-			f->type = tmp.type;
-			f->order = tmp.order;
+			f = new TargetField (tmp.name, tmp.type, tmp.encrypted, tmp.order);
 
 			target->fields.append(f);
 		}
@@ -340,11 +404,9 @@ void DetailWindow::emitCommitData (int a) {
 }
 
 DetailWindow::~DetailWindow () {
-	target->freeResources();
-
-	if (currentImageBackN) delete currentImageBackN;
-	if (currentImageN) delete currentImageN;
-
+	//target->freeResources();
+	//if (currentImageBackN) delete currentImageBackN;
+	//if (currentImageN) delete currentImageN;
 }
 
 void DetailWindow::saveTarget () {
@@ -380,48 +442,26 @@ void DetailWindow::saveTarget () {
 
 	for (int i = 0; i < target->fields.count(); i++) {
 		TargetField * field = target->fields[i];
-		//qDebug () << "getting field " << i;
+		qDebug () << "getting field " << i;
 		
 		field->name = labels[i]->text();
-		field->value = values[i]->text();
-	}
-/*
-	//qDebug () << "getting extra fields ";
-	int M = target->fields.count();
-	for (int i = 0; i < 5; i++) {
-		//qDebug () << "getting extra field " << i;
-		if (labels[i + M]->text().length () > 0) {
-			//qDebug () << "getting new extra field " << i;
-
-			TargetField * field = new TargetField ();
-			
-			field->id = -1;
-			field->name = labels[i + M]->text();
-			field->value = values[i + M]->text();
-			field->type = 1; // TODO
-			field->encrypted = 1; // TODO
-
-			this->target->fields.append(field);
+		if (field->type == TYPE_TEXT) {
+			//qDebug () << "getting TEX field " << i;
+			field->value = ((QTextEdit*)values[i])->toPlainText();
 		}
 		else {
-			if (values[i + M]->text().length () > 0) {
-				if (messageBox.isNull()) {
-					messageBox = new QMessageBox(this);
-					messageBox->information(this, "Wallet", "All fields must have a name", QMessageBox::Ok);
-				}
-				delete messageBox;
-				messageBox = 0;
-				return;
-			}
+			//qDebug () << "getting LIN field " << i;
+			field->value = ((QLineEdit*)values[i])->text();
 		}
 	}
-*/
+	qDebug () << "updating target";
 	if (isNew) {
 		parent->_insertTarget(this->target);
 	}
 	else {
 		parent->_updateTarget(this->target);
 	}
+	qDebug () << "updating target face";
 
 	if (deleteFace) {
 		qDebug () << "DELETE FACE";
@@ -440,6 +480,7 @@ void DetailWindow::saveTarget () {
 		}
 	}
 
+	qDebug () << "updating target back";
 
 	if (deleteBack) {
 		qDebug () << "DELETE BACK";
@@ -453,6 +494,13 @@ void DetailWindow::saveTarget () {
 		parent->_updateTargetPictureBack(target);
 	}
 
+	qDebug () << "end updating ";
+
+	target->freeResources();
+	if (currentImageBackN) delete currentImageBackN;
+	if (currentImageN) delete currentImageN;
+
+	qDebug () << "clse ";
 	close();
 }
 
@@ -479,11 +527,6 @@ void DetailWindow::_newTarget () {
 
 void DetailWindow::loadImage (QString q, QImage& res) {
 	QImageReader reader;
-	// Set image name
-/*	QString path(QDir::home().path());
-	path.append(QDir::separator()).append("photo.png");
-	path = QDir::toNativeSeparators(path);
-*/
 	reader.setFileName(q);
 	//Set/scale image size into screen
 	QSize imageSize = reader.size();
@@ -508,6 +551,7 @@ void DetailWindow::cardMenu(const QPoint & _p) {
 		QMenu menu(this);
 		if (edit) {
 			menu.addAction("Edit", this, SLOT(cardFace()));
+			menu.addAction("Mirror", this, SLOT(mirrorFace()));
 			menu.addAction("Delete", this, SLOT(deleteCardFace()));
 		}
 		else {
@@ -520,6 +564,7 @@ void DetailWindow::cardMenu(const QPoint & _p) {
 		QMenu menu(this);
 		if (edit) {
 			menu.addAction("Edit", this, SLOT(cardBack()));
+			menu.addAction("Mirror", this, SLOT(mirrorBack()));
 			menu.addAction("Delete", this, SLOT(deleteCardBack()));
 		}
 		else {
@@ -557,17 +602,12 @@ void DetailWindow::cardFace()
 	                this,
 	                "Choose card face image",
 	                path,
-	                "Images (*.png *.jpg)");
+	                "Images (*.png *.jpg *.gif *.jpeg)");
 
 	if (!s.isNull () && !s.isEmpty ()) {
 		//qDebug () << "loading " << s;
 		if (currentImageN) delete currentImageN;
 
-		/*QImage * tmp = new QImage (s);
-		currentImageN = new QImage ();
-		*currentImageN = tmp->scaled (INTERNAL_WIDTH, INTERNAL_HEIGH);
-		delete tmp;
-		*/
 		currentImageN = new QImage ();
 		loadImage(s, *currentImageN); //;
 		//qDebug () << "loaded " << s;
@@ -577,13 +617,46 @@ void DetailWindow::cardFace()
 		deleteFace = false;
 	}
 }
+
+void DetailWindow::mirrorFace () {
+	if (currentImageN) {
+		QImage * tmp = currentImageN;
+		currentImageN = new QImage (tmp->mirrored (true, true));
+		imageLabel->setPixmap(QPixmap::fromImage (*currentImageN));
+		delete tmp;
+		deleteFace = false;
+	}
+	else
+	if (currentImage) {
+		currentImageN = new QImage (currentImage->mirrored (true, true));
+		imageLabel->setPixmap(QPixmap::fromImage (*currentImageN));
+		deleteFace = false;
+	}
+}
+
+void DetailWindow::mirrorBack () {
+	if (currentImageBackN) {
+		QImage * tmp = currentImageBackN;
+		currentImageBackN = new QImage (tmp->mirrored (true, true));
+		imageBackLabel->setPixmap(QPixmap::fromImage (*currentImageBackN));
+		delete tmp;
+		deleteBack = false;
+	}
+	else
+	if (currentImageBack) {
+		currentImageBackN = new QImage (currentImageBack->mirrored (true, true));
+		imageBackLabel->setPixmap(QPixmap::fromImage (*currentImageBackN));
+		deleteBack = false;
+	}
+}
 void DetailWindow::cardBackMenu(const QPoint & pos) {
 
-	qDebug () << "MENU";
+	//qDebug () << "MENU";
 
 	QMenu menu(this);
 	if (edit) {
 		menu.addAction("Edit", this, SLOT(cardBack()));
+		menu.addAction("Mirror", this, SLOT(mirrorBack()));
 		menu.addAction("Delete", this, SLOT(deleteCardBack()));
 	}
 	else {
@@ -593,11 +666,12 @@ void DetailWindow::cardBackMenu(const QPoint & pos) {
 }
 void DetailWindow::cardFaceMenu(const QPoint & pos) {
 
-	qDebug () << "MENU";
+	//qDebug () << "MENU";
 
 	QMenu menu(this);
 	if (edit) {
 		menu.addAction("Edit", this, SLOT(cardFace()));
+		menu.addAction("Mirror", this, SLOT(mirrorFace()));
 		menu.addAction("Delete", this, SLOT(deleteCardFace()));
 	}
 	else {
@@ -634,16 +708,11 @@ void DetailWindow::cardBack()
 	                this,
 	                "Choose card back image",
 	                path,
-	                "Images (*.png *.jpg)");
+	                "Images (*.png *.jpg *.gif *.jpeg)");
 	if (!s.isNull () && !s.isEmpty ()) {
 		//qDebug () << "loading " << s;
 		if (currentImageBackN) delete currentImageBackN;
 		
-		/*QImage * tmp = new QImage (s);
-		currentImageBackN = new QImage ();
-		*currentImageBackN = tmp->scaled (INTERNAL_WIDTH, INTERNAL_HEIGH);
-		delete tmp;
-		*/
 		currentImageBackN = new QImage ();
 		loadImage(s, *currentImageBackN); //;
 		
@@ -652,3 +721,202 @@ void DetailWindow::cardBack()
 		deleteBack = false;
 	}
 }
+
+CreateFieldDialog::CreateFieldDialog(Target * target, DetailWindow *paTarget) :
+		QDialog((QWidget*)paTarget)
+{
+	pos = 0;
+	this->target = target;
+	this->parent = paTarget;
+
+	setContextMenuPolicy(Qt::NoContextMenu);
+
+	// Create dialog main layout
+
+	QHBoxLayout * layout = new QHBoxLayout(this);
+
+	layout->setMargin(0);
+	layout->setSpacing(0);
+
+	layout->setSpacing(10);
+	layout->setMargin(10);
+
+	QDialogButtonBox* dlgBtnBox = new QDialogButtonBox(this);
+	dlgBtnBox->setOrientation(Qt::Vertical);
+
+	previousButton = new QPushButton(QIcon(":/resources/icon_previous.png"), "Previous", this);
+	previousButton->setFixedHeight(70);
+	previousButton->setContextMenuPolicy(Qt::NoContextMenu);
+	previousButton->setFocusPolicy(Qt::NoFocus);
+	previousButton->setIconSize(QSize(48, 48));
+	QObject::connect(previousButton, SIGNAL(clicked()), this, SLOT(previous()));
+	dlgBtnBox->addButton(previousButton, QDialogButtonBox::ActionRole);
+
+	nextButton = new QPushButton(QIcon(":/resources/icon_next.png"), "Next", this);
+	nextButton->setFixedHeight(70);
+	nextButton->setContextMenuPolicy(Qt::NoContextMenu);
+	nextButton->setFocusPolicy(Qt::NoFocus);
+	nextButton->setIconSize(QSize(48, 48));
+	QObject::connect(nextButton, SIGNAL(clicked()), this, SLOT(next()));
+	dlgBtnBox->addButton(nextButton, QDialogButtonBox::ActionRole);
+
+	deleteButton = new QPushButton(QIcon(":/resources/icon_delete.png"), "Delete", this);
+	deleteButton->setFixedHeight(70);
+	deleteButton->setContextMenuPolicy(Qt::NoContextMenu);
+	deleteButton->setFocusPolicy(Qt::NoFocus);
+	deleteButton->setIconSize(QSize(48, 48));
+	QObject::connect(deleteButton, SIGNAL(clicked()), this, SLOT(deleteCurrent()));
+	dlgBtnBox->addButton(deleteButton, QDialogButtonBox::ActionRole);
+
+	QPushButton * newButton = new QPushButton(QIcon(":/resources/icon_add_new.png"), "Add new", this);
+	newButton->setFixedHeight(70);
+	newButton->setContextMenuPolicy(Qt::NoContextMenu);
+	newButton->setFocusPolicy(Qt::NoFocus);
+	newButton->setIconSize(QSize(48, 48));
+	QObject::connect(newButton, SIGNAL(clicked()), this, SLOT(addNew()));
+	dlgBtnBox->addButton(newButton, QDialogButtonBox::ActionRole);
+
+	QPushButton * closeButton = new QPushButton("Close", this);
+	closeButton->setFixedHeight(70);
+	closeButton->setContextMenuPolicy(Qt::NoContextMenu);
+	closeButton->setFocusPolicy(Qt::NoFocus);
+	closeButton->setIconSize(QSize(48, 48));
+	QObject::connect(closeButton, SIGNAL(clicked()), this, SLOT(save()));
+	dlgBtnBox->addButton(closeButton, QDialogButtonBox::ActionRole);
+
+	QGridLayout * componentLayout = new QGridLayout ();
+	componentLayout->setContentsMargins(15, 0, 15, 0);
+	componentLayout->setSpacing(5);
+
+	// Name label
+	QLabel * nameLabel = new QLabel("Name", this);
+	nameLabel->setContextMenuPolicy(Qt::NoContextMenu);
+	//nameLabel->setAlignment(Qt::AlignTop);
+	componentLayout->addWidget(nameLabel, 1, 1, 1, 2);
+
+// Name line edit
+	nameLineEdit = new QLineEdit(this);
+	nameLineEdit->setContextMenuPolicy(Qt::NoContextMenu);
+	//nameLineEdit->setAlignment(Qt::AlignTop);
+
+	componentLayout->addWidget(nameLineEdit, 1, 3, 1, 4);
+
+	// type label
+	QLabel * typeLabel = new QLabel("Type", this);
+	typeLabel->setContextMenuPolicy(Qt::NoContextMenu);
+	//typeLabel->setAlignment(Qt::AlignTop);
+	componentLayout->addWidget(typeLabel, 2, 1, 1, 2);
+
+	// type line edit
+	typeLineEdit = new QComboBox(this);
+	typeLineEdit->setContextMenuPolicy(Qt::NoContextMenu);
+	//typeLineEdit->setAlignment(Qt::AlignTop);
+
+	typeLineEdit->addItem ("String", QVariant(TYPE_STRING));
+	typeLineEdit->addItem ("Text", QVariant(TYPE_TEXT));
+	typeLineEdit->addItem ("Date", QVariant(TYPE_DATE));
+	typeLineEdit->addItem ("Number", QVariant(TYPE_INTEGER));
+	typeLineEdit->addItem ("Phone Number", QVariant(TYPE_PHONE_NUMBER));
+	typeLineEdit->addItem ("Password", QVariant(TYPE_PASSWORD));
+	typeLineEdit->addItem ("URL", QVariant(TYPE_URL));
+
+	componentLayout->addWidget(typeLineEdit, 2, 3, 1, 4);
+
+	// Name label
+	QLabel * orderLabel = new QLabel("Order", this);
+	orderLabel->setContextMenuPolicy(Qt::NoContextMenu);
+	//nameLabel->setAlignment(Qt::AlignTop);
+	componentLayout->addWidget(orderLabel, 3, 1, 1, 2);
+
+// Name line edit
+	orderEdit = new QLineEdit(this);
+	orderEdit->setContextMenuPolicy(Qt::NoContextMenu);
+	orderEdit->setValidator(new QIntValidator (1, 100, this));
+
+	componentLayout->addWidget(orderEdit, 3, 3, 1, 4);
+
+	layout->addLayout(componentLayout);
+	layout->setAlignment(componentLayout, Qt::AlignVCenter);
+
+	layout->addWidget(dlgBtnBox);
+	layout->setAlignment(dlgBtnBox, Qt::AlignBottom);
+
+	setLayout(layout);
+
+	showField ();
+}
+
+void CreateFieldDialog::update () {
+	if (pos < target->fields.length()) {
+		int type = typeLineEdit->itemData(typeLineEdit->currentIndex()).toInt();
+		int order = QVariant (orderEdit->text()).toInt();
+		target->fields[pos]->name = nameLineEdit->text();
+		target->fields[pos]->type = type;
+		target->fields[pos]->order = order;
+	}
+}
+
+void CreateFieldDialog::previous () {
+	if (pos > 0) {
+		update ();
+		pos--;
+		showField ();
+	}
+}
+void CreateFieldDialog::addNew () {
+	update ();
+	pos = target->fields.length();
+	target->fields.append(new TargetField ("", TYPE_STRING, true, 100));
+	showField ();
+}
+void CreateFieldDialog::next () {
+	if ((pos + 1) < target->fields.length()) {
+		update ();
+		pos++;
+		showField ();
+	}
+}
+void CreateFieldDialog::deleteCurrent () {
+	if (pos >= 0 && pos < target->fields.length()) {
+		update ();
+		target->fields.removeAt(pos);
+		if (pos >= target->fields.length()) {
+			pos--;
+			if (pos < 0) pos = 0;
+		}
+		showField ();
+	}
+}
+void CreateFieldDialog::showField () {
+	if (pos == 0) {
+		previousButton->setDisabled(true);
+	}
+	else {
+		previousButton->setDisabled(false);
+	}
+	if ((pos + 1) >= target->fields.length()) {
+		nextButton->setDisabled(true);
+	}
+	else {
+		nextButton->setDisabled(false);
+	}
+	if (pos >= 0 && pos < target->fields.length()) {
+		deleteButton->setDisabled(false);
+	}
+	else {
+		deleteButton->setDisabled(true);
+	}
+	if (pos < target->fields.length()) {
+		nameLineEdit->setText(target->fields[pos]->name);
+		int i = typeLineEdit->findData(QVariant (target->fields[pos]->type));
+		typeLineEdit->setCurrentIndex(i);
+		orderEdit->setText(QString::number(target->fields[pos]->order));
+	}
+}
+void CreateFieldDialog::save () {
+	update ();
+	close();
+}
+CreateFieldDialog::~CreateFieldDialog() {
+
+}
