@@ -6,8 +6,7 @@
 #include "LoginDialog.h"
 #include "Constants.h"
 
-LoginDialog::LoginDialog(eWallet *paTarget) :
-		QDialog((QWidget*)paTarget)
+LoginDialog::LoginDialog(eWallet *paTarget): QWidget((QWidget*)paTarget)
 {
 	setContextMenuPolicy(Qt::NoContextMenu);
 
@@ -15,12 +14,10 @@ LoginDialog::LoginDialog(eWallet *paTarget) :
 	QGridLayout * componentLayout = new QGridLayout ();
 	this->parent = paTarget;
 
-	layout->setMargin(0);
 	layout->setSpacing(0);
-#ifdef Q_WS_HILDON
-	layout->setSpacing(10);
 	layout->setMargin(10);
-#endif
+
+	componentLayout->setContentsMargins(20, 20, 20, 20);
 
 	QPushButton * imageButton [12];
 
@@ -35,12 +32,12 @@ LoginDialog::LoginDialog(eWallet *paTarget) :
 		QString text ('0' + i + 1);
 
 		if (i == 9) {
-			text = "Accept";
-			imageButton[i] = new QPushButton(QIcon(":/resources/icon_accept.png"), text, this);
+			text = "Ok";
+			imageButton[i] = new QPushButton(text, this); // QIcon(":/resources/icon_accept.png"),
 			imageButton[i] ->setFixedHeight(60);
 			imageButton[i] ->setContextMenuPolicy(Qt::NoContextMenu);
 			imageButton[i] ->setFocusPolicy(Qt::NoFocus);
-			imageButton[i] ->setIconSize(QSize(40, 40));
+			imageButton[i] ->setIconSize(QSize(20, 20));
 			componentLayout->addWidget(imageButton[i], i / 3, i % 3, 1, 1);
 
 			//signalMapper->setMapping(imageButton[i], text);
@@ -49,11 +46,11 @@ LoginDialog::LoginDialog(eWallet *paTarget) :
 		} else {
 			if (i == 10) {
 				text = "0";
-				imageButton[i] = new QPushButton(QIcon(":/resources/number.png"), text, this);
+				imageButton[i] = new QPushButton(text, this); // QIcon(":/resources/number.png"),
 				imageButton[i] ->setFixedHeight(60);
 				imageButton[i] ->setContextMenuPolicy(Qt::NoContextMenu);
 				imageButton[i] ->setFocusPolicy(Qt::NoFocus);
-				imageButton[i] ->setIconSize(QSize(40, 40));
+				imageButton[i] ->setIconSize(QSize(20, 20));
 				componentLayout->addWidget(imageButton[i], i / 3, i % 3, 1, 1);
 
 				signalMapper->setMapping(imageButton[i], text);
@@ -62,22 +59,22 @@ LoginDialog::LoginDialog(eWallet *paTarget) :
 			} else
 				if (i == 11) {
 					text = "Clear";
-					imageButton[i] = new QPushButton(QIcon(":/resources/icon_clear.png"), text, this);
+					imageButton[i] = new QPushButton(text, this); // QIcon(":/resources/icon_clear.png"),
 					imageButton[i] ->setFixedHeight(60);
 					imageButton[i] ->setContextMenuPolicy(Qt::NoContextMenu);
 					imageButton[i] ->setFocusPolicy(Qt::NoFocus);
-					imageButton[i] ->setIconSize(QSize(40, 40));
+					imageButton[i] ->setIconSize(QSize(20, 20));
 					componentLayout->addWidget(imageButton[i], i / 3, i % 3, 1, 1);
 
 					//signalMapper->setMapping(imageButton[i], text);
 
 					QObject::connect(imageButton[i], SIGNAL(clicked()), this, SLOT(clearS()));
 				} else {
-					imageButton[i] = new QPushButton(QIcon(":/resources/number.png"), text, this);
+					imageButton[i] = new QPushButton(text, this); // QIcon(":/resources/number.png"),
 					imageButton[i] ->setFixedHeight(60);
 					imageButton[i] ->setContextMenuPolicy(Qt::NoContextMenu);
 					imageButton[i] ->setFocusPolicy(Qt::NoFocus);
-					imageButton[i] ->setIconSize(QSize(40, 40));
+					imageButton[i] ->setIconSize(QSize(20, 20));
 					componentLayout->addWidget(imageButton[i], i / 3, i % 3, 1, 1);
 
 					signalMapper->setMapping(imageButton[i], text);
@@ -87,14 +84,24 @@ LoginDialog::LoginDialog(eWallet *paTarget) :
 		}
 	}
 
+	componentLayout->addItem(new QSpacerItem(2,3), 4, 3, 3, 2);
+
 	connect(signalMapper, SIGNAL(mapped(const QString &)),
 	        this, SLOT(select(const QString &)));
 
-	componentLayout->addWidget(label, 4, 0, 1, 3);
+	componentLayout->addWidget(label, 4, 0, 1, 4);
 
 	QDialogButtonBox* dlgBtnBox = new QDialogButtonBox(this);
 	dlgBtnBox->setOrientation(Qt::Vertical);
 
+	QPushButton * aboutButton = new QPushButton(QIcon(":/resources/about.png"), "About", this);
+	aboutButton->setFixedHeight(70);
+	aboutButton->setContextMenuPolicy(Qt::NoContextMenu);
+	aboutButton->setFocusPolicy(Qt::NoFocus);
+	aboutButton->setIconSize(QSize(48, 48));
+	QObject::connect(aboutButton, SIGNAL(clicked()), this, SLOT(about()));
+	dlgBtnBox->addButton(aboutButton, QDialogButtonBox::ActionRole);
+
 	QPushButton * deleteButton = new QPushButton(QIcon(":/resources/exit.png"), "Exit", this);
 	deleteButton->setFixedHeight(70);
 	deleteButton->setContextMenuPolicy(Qt::NoContextMenu);
@@ -116,7 +123,19 @@ LoginDialog::LoginDialog(eWallet *paTarget) :
 LoginDialog::~LoginDialog()
 {
 }
-
+void LoginDialog::about()
+{
+	if (aboutDialogDlg.isNull()) {
+		aboutDialogDlg = new AboutDialog(this);
+		aboutDialogDlg->showMaximized();
+#ifdef Q_WS_MAEMO_5
+		aboutDialogDlg->setAttribute(Qt::WA_Maemo5StackedWindow);
+#endif
+		aboutDialogDlg->setAttribute(Qt::WA_DeleteOnClose);
+        //connect(detailWindowEdit, SIGNAL(destroyed(QObject*)), this, SLOT(windowDestroyed(QObject*)));
+        aboutDialogDlg->show();
+	}
+}
 void LoginDialog::clearS()
 {
 	label->setText ("");
@@ -125,9 +144,10 @@ void LoginDialog::clearS()
 
 void LoginDialog::acceptS() {
 	if (parent->openDataBase (current)) {
-		close();
+		//close();
 	}
 	else {
+
 		label->setText ("");
 		current = "";
 	}
@@ -174,4 +194,4 @@ void LoginDialog::focusOutEvent ( QFocusEvent * event ) {
 	
 	qDebug () << "LOST FOCUS";
 }
-*/
\ No newline at end of file
+*/
