@@ -1,29 +1,68 @@
 /*
  * Copyright (c) 2010 David Galindo <nowheremanmail@gmail.com>
+ *
+// TODO  several files? or several wallets in database?
+// TODO export to a file with pictures!!!??
+// TODO import picture from camera
+// TODO modify picture (zoom, ...)
+// TODO support for portaint
+// TODO final images
+ * TODO strong encryption with characters
+ * TODO core dump when clicked normal key on login
+ *
+ *
  */
 
 #include "eWallet.h"
+#include "Constants.h"
 #include "DatabaseManager.h"
 #include <Qt>
 #include <QFile>
-#ifdef Q_WS_MAEMO_5
-//#include <QtMaemo5/QMaemo5KineticScroller>
-#endif
+#include <QDBusInterface>
+#include <QDBusConnection>
+//#ifdef Q_WS_MAEMO_5
+//#include <mce/mode-names.h>
+//#include <mce/dbus-names.h>
+//#endif
 
 DatabaseManager * dbManager;
 
 eWallet::eWallet(QWidget *parent) :
 		QMainWindow(parent)
 {
+
+//qDBusRegisterMetaType<Property>();
+//qDBusRegisterMetaType< QList<Property> >();
+	loadTypesAndFields ();
+
 	::dbManager = new DatabaseManager(this);
 
 	askPassword ();
-	
+
+//	QDBusInterface *interface = new QDBusInterface(MCE_SERVICE, MCE_REQUEST_PATH,
+//                                   MCE_REQUEST_IF, QDBusConnection::systemBus(),
+//                                   this);
+//
+//    QDBusMessage reply = interface->call(MCE_DISPLAY_SIG);
+//    if (reply.type() == QDBusMessage::ErrorMessage)
+//        qDebug() << "error? " << reply.errorMessage();
+
+	QDBusConnection conn = QDBusConnection::systemBus(); //QString(MCE_SERVICE), QString(MCE_SIGNAL_PATH)
+	bool success = conn.connect("","", QString("com.nokia.mce.signal"/*MCE_SIGNAL_IF*/), QString("display_status_ind"/*MCE_DISPLAY_SIG*/) , this, SLOT(listenInactivity(QString)));
+//	bool success = conn.connect(QString(MCE_SERVICE), QString(MCE_SIGNAL_PATH), QString(MCE_SIGNAL_IF), QString(MCE_DISPLAY_SIG) /*MCE_DISPLAY_SIG MCE_DEVLOCK_MODE_GET MCE_DISPLAY_STATUS_GET MCE_INACTIVITY_STATUS_GET*/, this, SLOT(listenInactivity(QString)));
+//	//bool success = conn.connect("com.nokia.mce","/com/nokia/mce/signal","com.nokia.mce.signal", MCE_TKLOCK_MODE_GET, this, SLOT(listenInactivity(string)));
+
+	qDebug() << "SUCCESS : " << success;
+
+	//timer = new QTimer(this);
+	//connect(timer, SIGNAL(timeout()), this, SLOT(lockWallet()));
+	//timer->start(5000);
+	//showList  ();
+}
+
+void eWallet::showList () {
 	setProperty("FingerScrollable", false);
 	//setContextMenuPolicy(Qt::NoContextMenu);
-
-	loadTypesAndFields ();
-
 	//                         // Central widget and Layout
 	centralWidget = new QWidget(this);
 
@@ -37,6 +76,26 @@ eWallet::eWallet(QWidget *parent) :
 	fnListWidget = new QFNListWidget(this);
 	vboxlayout->addWidget(fnListWidget);
 
+	inputWidget = new QHBoxLayout;
+	inputWidget->setContentsMargins(2, 2, 2, 0);
+	vboxlayout->addLayout(inputWidget);
+
+	cancelSearch = new QPushButton(QIcon(":/resources/icon_clear.png"), "", this);
+	searchField = new QLineEdit ();
+	hideSearch = new QPushButton(QIcon(":/resources/icon_hide.png"), "", this);
+
+	inputWidget->addWidget(cancelSearch);
+	inputWidget->addWidget(searchField);
+	inputWidget->addWidget(hideSearch);
+
+	QObject::connect(cancelSearch, SIGNAL(clicked()), this, SLOT(_cancelSearch()));
+	QObject::connect(hideSearch, SIGNAL(clicked()), this, SLOT(_hideSearch()));
+	QObject::connect(searchField, SIGNAL(textChanged(const QString &)), this, SLOT(textChanged(const QString &)));
+
+	cancelSearch->hide ();
+	searchField->hide ();
+	hideSearch->hide ();
+
 	showTargets ();
 
 	centralWidget->setLayout(vboxlayout);
@@ -45,11 +104,61 @@ eWallet::eWallet(QWidget *parent) :
 	createMenu ();
 
 	centralWidget->setFocus ();
-	
+}
 
-	timer = new QTimer(this);
-	//TODO connect(timer, SIGNAL(timeout()), this, SLOT(lockWallet()));
-    //TODO timer->start(5000);
+void eWallet::listenInactivity(QString s) {
+	//qDebug () << "HELLO " << s;
+	if (s == QString("off") && ::dbManager->isReady ()) {
+		::dbManager->lock ();
+
+		// TODO sure????
+		fnListWidget->clear();
+		list.clear ();
+
+		if (!detailWindow.isNull()) {
+			detailWindow->close();
+			delete detailWindow;
+			detailWindow = 0;
+	    }
+
+	    if (!detailWindowEdit.isNull()) {
+			detailWindowEdit->close();
+			delete detailWindowEdit;
+			detailWindowEdit = 0;
+	    }
+
+	}
+	else {
+		if (s == QString("on") && !::dbManager->isReady ()) {
+			askPassword ();
+//			if (::dbManager->isReady ()) {
+//				showList ();
+//			}
+		}
+	}
+}
+
+void eWallet::textChanged (const QString & text) {
+	//qDebug () << text;
+
+	filterBy (text);
+}
+
+void eWallet::filterBy (const QString & s) {
+	fnListWidget->filterBy (s);
+}
+
+void eWallet::_cancelSearch () {
+	searchField->setText("");
+	searchField->hide();
+	cancelSearch->hide();
+	hideSearch->hide ();
+}
+
+void eWallet::_hideSearch () {
+	searchField->hide();
+	cancelSearch->hide();
+	hideSearch->hide ();
 }
 
 void eWallet::lockWallet () {
@@ -59,14 +168,43 @@ void eWallet::lockWallet () {
 
 bool eWallet::openDataBase (QString pas)
 {
-	return ::dbManager->openDB(pas);
+	bool res = ::dbManager->openDB(pas);
+
+	if (!res) {
+		if (++times > 3) {
+			exit(3);
+		}
+		else {
+			if (!::dbManager->isReady ()) {
+				if (messageBox.isNull()) {
+					messageBox = new QMessageBox(this);
+					messageBox->information(this, "Wallet", "Wrong password", QMessageBox::Ok);
+				}
+				delete messageBox;
+				messageBox = 0;
+			}
+		}
+	}
+	else {
+		showList ();
+	}
+	return res;
 }
 
 eWallet::~eWallet ()
 {
 	delete ::dbManager;
 }
-
+/*
+ * TYPES
+ * 	S -> String
+ * 	T -> Text
+ * 	D -> Date
+ *  U -> URL
+ *  P -> Password
+ *  N -> Number
+ *  H -> Phone number
+ */
 void eWallet::loadTypesAndFields () {
 	  QFile file(":/resources/types.txt");
 	  QString line;
@@ -83,14 +221,51 @@ void eWallet::loadTypesAndFields () {
 
 	      if (line.length() > 0) {
 	    	  if ((i++) == 0) {
-	    		  this->types.append(line);
+	    		  int p = line.indexOf(';');
+	    		  if (p >= 0) {
+	    			  // TODO
+	    			  QChar imageType = line.at(p + 1);
+					  this->types.append(line.mid(0, p).trimmed());
+	    		  }
+	    		  else {
+					  this->types.append(line);
+	    		  }
 	    		  this->fieldsByType.append (new QList<TargetField>);
-				  qDebug () << " new type "<< line << " " << this->types.length();
+				  //qDebug () << " new type "<< line << " " << this->types.length();
 	    	  }
 	    	  else {
-	    		  // TODO implements types
-	    		  this->fieldsByType [this->types.length() - 1]->append(TargetField (line, TYPE_STRING, true, i));
-				  qDebug () << " new field "<< line << " " << this->types.length();
+	    		  int p = line.indexOf(';');
+	    		  if (p >= 0) {
+	    			  int type = TYPE_STRING;
+	    			  switch (line.at(p + 1).toAscii()) {
+						  case 'T':
+							  type = TYPE_TEXT;
+							  break;
+						  case 'D':
+							  type = TYPE_DATE;
+							  break;
+						  case 'U':
+							  type = TYPE_URL;
+							  break;
+						  case 'P':
+							  type = TYPE_PASSWORD;
+							  break;
+						  case 'N':
+							  type = TYPE_INTEGER;
+							  break;
+						  case 'H':
+							  type = TYPE_PHONE_NUMBER;
+							  break;
+						  case 'S':
+						  default:
+			    			  type = TYPE_STRING;
+						}
+	    			  this->fieldsByType [this->types.length() - 1]->append(TargetField (line.mid(0, p).trimmed(), type, true, i));
+	    		  }
+	    		  else {
+	    			  this->fieldsByType [this->types.length() - 1]->append(TargetField (line, TYPE_STRING, true, i));
+	    		  }
+				  //qDebug () << " new field "<< line << " " << this->types.length();
 	    	  }
 	      }
 	      else {
@@ -104,74 +279,24 @@ void eWallet::loadTypesAndFields () {
 	  }
 }
 
-//
-//	this->types.append("Credit Card");
-//	this->types.append("Membership Info");
-//	this->types.append("Password");
-//	this->types.append("Bank Account");
-//	this->types.append("Cellular Phone");
-//	this->types.append("Web Site");
-//	this->types.append("Health Numbers");
-//	this->types.append("Passport Info");
-//	this->types.append("Note Card");
-//	this->types.append("Car Info");
-//	this->types.append("ID Card");
-//	this->types.append("Social Security Number");
-//	this->types.append("Driver's License");
-//	this->types.append("Email Account");
-//	this->types.append("Others");
-//
-//	fieldsByType [0].append(TargetField ("Expired Date", TYPE_DATE, true, 1));
-//	fieldsByType [0].append(TargetField ("Bank", TYPE_STRING, true, 2));
-//	fieldsByType [0].append(TargetField ("PIN", TYPE_INTEGER, true, 3));
-//	fieldsByType [0].append(TargetField ("Extra 1", TYPE_STRING, true, 40));
-//	fieldsByType [0].append(TargetField ("Extra 2", TYPE_STRING, true, 50));
-//	fieldsByType [0].append(TargetField ("Extra 3", TYPE_STRING, true, 60));
-//	fieldsByType [0].append(TargetField ("Extra 4", TYPE_STRING, true, 70));
-//	fieldsByType [0].append(TargetField ("Extra 5", TYPE_STRING, true, 80));
-//
-//	fieldsByType [1].append(TargetField ("Issued Date", TYPE_DATE, true, 1));
-//	fieldsByType [1].append(TargetField ("Titular", TYPE_STRING, true, 2));
-//	fieldsByType [1].append(TargetField ("PIN", TYPE_INTEGER, true, 3));
-//	fieldsByType [1].append(TargetField ("Extra 1", TYPE_STRING, true, 40));
-//	fieldsByType [1].append(TargetField ("Extra 2", TYPE_STRING, true, 50));
-//	fieldsByType [1].append(TargetField ("Extra 3", TYPE_STRING, true, 60));
-//	fieldsByType [1].append(TargetField ("Extra 4", TYPE_STRING, true, 70));
-//	fieldsByType [1].append(TargetField ("Extra 5", TYPE_STRING, true, 80));
-//
-//	fieldsByType [2].append(TargetField ("Extra 1", TYPE_STRING, true, 40));
-//	fieldsByType [2].append(TargetField ("Extra 2", TYPE_STRING, true, 50));
-//	fieldsByType [2].append(TargetField ("Extra 3", TYPE_STRING, true, 60));
-//	fieldsByType [2].append(TargetField ("Extra 4", TYPE_STRING, true, 70));
-//	fieldsByType [2].append(TargetField ("Extra 5", TYPE_STRING, true, 80));
+void eWallet::timerEvent(QTimerEvent* event) {
 
-
-
-void eWallet::timerEvent(QTimerEvent* event)
-{
 }
 void eWallet::keyPressEvent(QKeyEvent* event) {
 	//qDebug()<<"key pressed " << event->key();
-
-
 	
-/*		if (event->key() == Qt::Key_Backspace || event->key() == Qt::Key_Delete) {
-			if (current.size () > 0) {
-				current = current.left (current.size ()-1);
-				label->setText (QString ("************************").right(current.size()));						
-			}
-			event->accept();
-		}
-		else {
-			if (event->key() == Qt::Key_Return || event->key() == Qt::Key_Enter) {
-				event->accept();
-				acceptS();
-			}
-			else {*/
-				QWidget::keyPressEvent(event);
-/*			}
-		}
-	}*/
+	if (searchField->isHidden()) {
+		cancelSearch->show ();
+		searchField->show ();
+		hideSearch->show ();
+		searchField->setFocus();
+
+		QCoreApplication::sendEvent (searchField, event);
+
+		event->accept();
+	}
+	//timer->stop();
+	//timer->start(5000);
 }
 
 
@@ -205,6 +330,8 @@ void eWallet::createMenu()
 {
 	menuBar()->addAction("New", this, SLOT(newTarget()));
 	menuBar()->addAction("Import", this, SLOT(import()));
+	menuBar()->addAction("Export", this, SLOT(exportData()));
+	menuBar()->addAction("About", this, SLOT(about()));
 	menuBar()->addAction("Exit", QApplication::instance(), SLOT(quit()));
 
 }
@@ -287,7 +414,7 @@ bool eWallet::_updateTargetPicture(Target* target)
 {
 	if (::dbManager->updateTargetPicture(target)) {
 		if (row >= 0) {
-			fnListWidget->itemAt (row)->setImage (target->picture, target->type);
+			fnListWidget->itemAt (row)->setImage (target->getPictureSmall(), target->type);
 		}
 		return true;
 	}
@@ -303,7 +430,7 @@ bool eWallet::_updateTargetPictureBack(Target* target)
 
 void eWallet::deleteTarget(Target * target) {
 
-	qDebug()<<"delete ";
+	//qDebug()<<"delete ";
 
 	row = fnListWidget->findRow(target);
 	
@@ -329,7 +456,7 @@ void eWallet::windowDestroyed(QObject* obj)
 }
 
 void eWallet::viewTarget(Target * rent) {
-	qDebug()<<"view ";
+	//qDebug()<<"view ";
 
 	row = fnListWidget->findRow(rent);
 
@@ -353,7 +480,7 @@ void eWallet::viewTarget(Target * rent) {
 }
 
 void eWallet::editTarget(Target * rent) {
-	qDebug()<<"edit ";
+	//qDebug()<<"edit ";
 
 	row = fnListWidget->findRow(rent);
 
@@ -374,7 +501,7 @@ void eWallet::editTarget(Target * rent) {
 }
 
 void eWallet::import() {
-	QString path(QDesktopServices::storageLocation(QDesktopServices::PicturesLocation));
+	QString path(QDesktopServices::storageLocation(QDesktopServices::HomeLocation));
 	QString s = QFileDialog::getOpenFileName(
 	                this,
 	                "Choose import file",
@@ -415,8 +542,37 @@ void eWallet::import() {
 						  TargetField * field = new TargetField ();
 						  field->name = line.mid (0, j).trimmed();
 						  field->value = line.mid(j + 1).trimmed();
+						  field->order = 10 * (i - 2);
 						  field->encrypted = true;
-						  field->type = TYPE_STRING;
+						  if (field->name.contains("url", Qt::CaseInsensitive)) {
+							  field->type = TYPE_URL;
+						  }
+						  else
+						  if (field->name.contains("password", Qt::CaseInsensitive)) {
+							  field->type = TYPE_PASSWORD;
+						  }
+						  else
+						  if (field->name.contains("pin", Qt::CaseInsensitive)) {
+							  field->type = TYPE_PASSWORD;
+						  }
+						  else
+						  if (field->name.contains("phone", Qt::CaseInsensitive)) {
+							  field->type = TYPE_PHONE_NUMBER;
+						  }
+						  else
+						  if (field->name.contains("number", Qt::CaseInsensitive)) {
+							  field->type = TYPE_INTEGER;
+						  }
+						  else
+						  if (field->name.contains("date", Qt::CaseInsensitive)) {
+							  field->type = TYPE_DATE;
+						  }
+						  else
+						  if (field->name.contains("note", Qt::CaseInsensitive)) {
+							  field->type = TYPE_TEXT;
+						  }
+						  else
+							  field->type = TYPE_STRING;
 						  target->fields.append(field);
 					  }
 
@@ -451,7 +607,7 @@ void eWallet::import() {
 }
 
 void eWallet::newTarget() {
-	qDebug()<<"new ";
+	//qDebug()<<"new ";
 
 	row = -1;
 
@@ -506,6 +662,45 @@ void eWallet::editTarget(Target * rent)
 	}
 }
 */
+
+void eWallet::exportData() {
+	QString path(QDesktopServices::storageLocation(QDesktopServices::HomeLocation));
+	QString s = QFileDialog::getSaveFileName(
+	                this,
+	                "Choose export file",
+	                path,
+	                "Text (*.txt)");
+	if (!s.isNull () && !s.isEmpty ()) {
+		  QFile file(s);
+
+		  if ( file.open(QIODevice::Text | QIODevice::WriteOnly) ) {
+		    // file opened successfully
+		    QTextStream t(&file);        // use a text stream
+
+			for (int i = 0; i < list.size (); i++) {
+				Target * target = list[i];
+
+				t << "Name: " << target->name << "\n";
+				t << "Type: " << target->type << "\n";
+
+				dbManager->getFields(target);
+
+				TargetField * field;
+				foreach(field, target->fields) {
+					t << field->name << ": " << field->value << "\n";
+				}
+
+				t << "\n\n";
+
+				target->freeFieldList();
+
+			}
+
+		    // Close the file
+		    file.close();
+		  }
+	}
+}
 void eWallet::_editcTarget() {
 	editTarget(this->fnListWidget->itemAt(row)->getSource());
 }
@@ -518,44 +713,98 @@ void eWallet::_viewcTarget() {
 
 //eWallet closeEvent function:
 void eWallet::closeEvent(QCloseEvent *event){
-	qDebug () << "close Event";
+	//qDebug () << "close Event";
         //saveSettings();
         event->accept();
 }
 
 void eWallet::askPassword () {
-	int times = 0;
+	times = 0;
 	
-	do {
+	menuBar()->clear();
+
+//	do {
 		if (loginDialogDlg.isNull()) {
 			loginDialogDlg = new LoginDialog(this);
-	#if defined(Q_OS_WIN32)
-			loginDialogDlg->setFixedSize(640,480);
-	#elif defined(Q_WS_HILDON)
-			loginDialogDlg->setMinimumSize(size().width(),size().height()-100);
-	#else
-			loginDialogDlg->showMaximized();
-	#endif
-			loginDialogDlg->exec();
-			delete loginDialogDlg;
-			loginDialogDlg = 0;
+			setCentralWidget(loginDialogDlg);
+			loginDialogDlg->setFocus ();
+//	#if defined(Q_OS_WIN32)
+//			loginDialogDlg->setFixedSize(640,480);
+//	#elif defined(Q_WS_HILDON)
+//			loginDialogDlg->setMinimumSize(size().width(),size().height()-100);
+//	#else
+//			loginDialogDlg->showMaximized();
+//	#endif
+//			loginDialogDlg->setFocus();
+//			loginDialogDlg->exec();
+//			delete loginDialogDlg;
+//			loginDialogDlg = 0;
 		}
 		
 		//qDebug () << "TIME " << times;
 			
-		if (++times > 3) {
-			exit(3);
-		}
-		else {
-			if (!::dbManager->isReady ()) {
-				if (messageBox.isNull()) {
-					messageBox = new QMessageBox(this);
-					messageBox->information(this, "Wallet", "Wrong password", QMessageBox::Ok);
-				}
-				delete messageBox;
-				messageBox = 0;
-			}
-		}
-	}
-	while(!::dbManager->isReady ());
+//		if (++times > 3) {
+//			exit(3);
+//		}
+//		else {
+//			if (!::dbManager->isReady ()) {
+//				if (messageBox.isNull()) {
+//					messageBox = new QMessageBox(this);
+//					messageBox->information(this, "Wallet", "Wrong password", QMessageBox::Ok);
+//				}
+//				delete messageBox;
+//				messageBox = 0;
+//			}
+//		}
+//	}
+//	while(!::dbManager->isReady ());
 }
+
+void eWallet::about() {
+	if (aboutDialogDlg.isNull()) {
+		aboutDialogDlg = new AboutDialog(this);
+		aboutDialogDlg->showMaximized();
+#ifdef Q_WS_MAEMO_5
+		aboutDialogDlg->setAttribute(Qt::WA_Maemo5StackedWindow);
+#endif
+		aboutDialogDlg->setAttribute(Qt::WA_DeleteOnClose);
+        //connect(detailWindowEdit, SIGNAL(destroyed(QObject*)), this, SLOT(windowDestroyed(QObject*)));
+        aboutDialogDlg->show();
+	}
+}
+AboutDialog::AboutDialog(QWidget *paTarget) : QMainWindow(paTarget)
+{
+	setContextMenuPolicy(Qt::NoContextMenu);
+
+	QWidget * centralWidget = new QWidget(this);
+	setCentralWidget(centralWidget);
+
+	QVBoxLayout * layout = new QVBoxLayout ();
+
+	QLabel * label;
+
+	label = new QLabel("<qt><br><br><p><i><b>qtWallet</b></i>, v.1.0</p>"
+			"<p>Developed by <b>nowheremanmail@gmail.com</b></p>"
+			"<br><br><br>"
+			"<p>if you think that qtWallet is useful for you, please</p></qt>", this);
+	label->setContextMenuPolicy(Qt::NoContextMenu);
+	label->setAlignment(Qt::AlignCenter);
+	layout->addWidget(label);
+
+	label = new QLabel("<qt><a href=\"https://www.paypal.com/cgi-bin/webscr?cmd=_donations&business=nowheremanmail@gmail.com&lc=US&item_name=qtWallet&currency_code=EUR&bn=PP%2dDonationsBF%3abtn_donate_SM%2egif%3aNonHosted\">DONATE</a></qt>", this);
+	label->setContextMenuPolicy(Qt::NoContextMenu);
+	label->setOpenExternalLinks (true);
+	label->setAlignment(Qt::AlignCenter);
+
+	QPixmap available(":/resources/donate.gif");
+	label->setPixmap(available);
+
+	layout->addWidget(label);
+
+	centralWidget->setLayout(layout);
+}
+
+AboutDialog::~AboutDialog()
+{
+}
+
