@@ -14,13 +14,17 @@
 #include "DetailedWindow.h"
 #include "LoginDialog.h"
 
-#define TYPE_TEXT 3
-#define TYPE_DATE 2
-#define TYPE_STRING 0
-#define TYPE_INTEGER 1
-
 extern DatabaseManager * dbManager;
 
+class AboutDialog: public QMainWindow
+{
+	Q_OBJECT
+
+public:
+	AboutDialog(QWidget * paTarget);
+	~AboutDialog();
+};
+
 class eWallet: public QMainWindow
 {
 	Q_OBJECT
@@ -29,11 +33,7 @@ public:
 	eWallet (QWidget *parent = 0);
 	~eWallet ();
 
-	void keyPressEvent(QKeyEvent*);
-	void timerEvent(QTimerEvent*);
-
 	bool openDataBase (QString pas);
-
 	bool _deleteTarget(Target* target);
 	bool _updateTarget(Target* target);
 	bool _updateTargetPicture(Target* target);
@@ -45,15 +45,19 @@ public:
 
 public slots:
 	void newTarget();
-	void import();
 	void editTarget(Target * t);
 	void deleteTarget(Target * t);
 	void viewTarget(Target * t);
+	Q_SCRIPTABLE void listenInactivity(QString s);
 
 private:
+	void showList ();
+	void keyPressEvent(QKeyEvent*);
+	void timerEvent(QTimerEvent*);
 	void createMenu();
 	void showTargets();
 	void loadTypesAndFields ();
+	void filterBy (const QString & s);
 	//void contextMenuEvent (QContextMenuEvent * event);
 
 private slots:
@@ -62,19 +66,31 @@ private slots:
 	void _deletecTarget();
 	void _viewcTarget();
 	void lockWallet ();
+	void _cancelSearch ();
+	void _hideSearch ();
+	void about ();
+	void import();
+	void exportData();
+	void textChanged (const QString &);
 	
 private:
 	//void timerEvent(QTimerEvent *event);
 	void askPassword ();
 	void closeEvent(QCloseEvent *event);
 	
-	QVBoxLayout* vboxlayout;
-
+	QVBoxLayout * vboxlayout;
+	int times;
 	QFNListWidget* fnListWidget;
 	QList<Target *> list;
 
+	QHBoxLayout * inputWidget;
+	QPushButton * cancelSearch;
+	QPushButton * hideSearch;
+	QLineEdit * searchField;
+
 	QPointer<CreateTargetDialog> createTargetDlg;
 	QPointer<LoginDialog> loginDialogDlg;
+	QPointer<AboutDialog> aboutDialogDlg;
 	QPointer<DetailWindow> detailWindow;
 	QPointer<DetailWindow> detailWindowEdit;
 	QWidget* centralWidget;
@@ -86,4 +102,6 @@ private:
 	
 	QTimer *timer;
 };
+
+
 #endif // EWALLET_H
